package jp.co.nanoconnect.boundball;

import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.util.AttributeSet;
import android.view.SurfaceHolder;
import android.view.SurfaceView;

public class BoundBallView extends SurfaceView implements SurfaceHolder.Callback {
	/** {[̔a */
	private static final int BALL_RADIUS = 35;
	/** {[F */
	private static final int COLOR_BALL = 0xFFB8C749;
	/** wiF */
	private static final int COLOR_BACKGROUND = 0xFFEFFEEF;	
	/** CW */
	private static final float COEFFICIENT_FRICTION = 0.2f;
	/** W */
	private static final float COEFFICIENT_RESTITUTION = 0.8f;

	
	/** `Xbh */
	private Thread mDrawThread;
	/** Xbh̃C[vtO */
	private boolean mRun;
	
	/** T[tF[X̕ */
	private int mSurfaceWidth;
	/** T[tF[X̍ */
	private int mSurfaceHeight;
	/** {[IuWFNg */
	private Ball mBall;
	/** wi̕`X^C */
	private Paint mBgStyle = new Paint();
	/** xZT̒l */
	private float[] mAcceleration;
	
	/* RXgN^ */
	public BoundBallView(Context context, AttributeSet attrs) {
		super(context, attrs);
		SurfaceHolder holder = getHolder();
		holder.addCallback(this);
	}

	/* T[tF[XɕύXƂɌĂ΂郁\bh */
	@Override
	public void surfaceChanged(SurfaceHolder holder, int format, int width,
			int height) {
		
		this.mSurfaceWidth = width;
		this.mSurfaceHeight = height;
		
		/* {[̐ */
		float x = mSurfaceWidth / 2;
		float y = mSurfaceHeight / 2;
		mBall = new Ball(x, y, BALL_RADIUS, COLOR_BALL);
			
		/* wi̕` */
		mBgStyle.setColor(COLOR_BACKGROUND);
		
		mRun = true;
		mDrawThread = new Thread(new Runnable() {
			@Override
			public void run() {
				Canvas canvas = null;
				SurfaceHolder holder = getHolder();
				/* C[v */
				while (mRun) {
					doUpdate();
					try {
						canvas = holder.lockCanvas();
						if (canvas != null) {
							doDraw(canvas);
						}
					} finally {
						if (canvas != null) {
							holder.unlockCanvasAndPost(canvas);
						}
					}
				}
			}
		});
		mDrawThread.start();
	}

	/* T[tF[XꂽƂɌĂ΂郁\bh */
	@Override
	public void surfaceCreated(SurfaceHolder holder) {
		/* NOP */
	}

	/* T[tF[XjƂɂ΂郁\bh */
	@Override
	public void surfaceDestroyed(SurfaceHolder holder) {
		/* Xbh̒~ */
		boolean retry = true;
		mRun = false;
        while (retry) {
            try {
                mDrawThread.join();
                retry = false;
            } catch (InterruptedException e) {
            	/* NOP */
            }
        }
	}
	
	/**
	 * ZT̒lZbg郁\bh
	 * @param acceleration@xZT̒lz
	 */
	public void setAcceleration(float[] acceleration) {
		this.mAcceleration = acceleration;
	}
	
	/* LoXɕ`s\bh */
	private void doDraw(Canvas canvas) {
		canvas.drawPaint(mBgStyle);
		canvas.drawCircle(mBall.mPx, mBall.mPy, mBall.mRadius, mBall.mPaint);
	}

	/* {[̍WXV郁\bh */
	private void doUpdate() {
		if (mAcceleration == null) {
			return;
		}
		/* {[̈ړ */
		final Ball b = mBall;
		b.mDx = b.mDx - (mAcceleration[0] * COEFFICIENT_FRICTION);
		b.mDy = b.mDy + (mAcceleration[1] * COEFFICIENT_FRICTION);

		b.mPx = b.mPx + b.mDx;
		b.mPy = b.mPy + b.mDy;

		/* ǖʏՓˎ̏ (X) */
		if (b.mPx < b.mRadius) {
			b.mPx = b.mRadius;
			b.mDx = b.mDx * (-COEFFICIENT_RESTITUTION);
		} else if (mSurfaceWidth - b.mRadius < b.mPx) {
			b.mPx = mSurfaceWidth - b.mRadius;
			b.mDx = b.mDx * (-COEFFICIENT_RESTITUTION);
		}

		/* ǖʏՓˎ̏ (Y) */
		if (b.mPy < b.mRadius) {
			b.mPy = b.mRadius;
			b.mDy = b.mDy * (-COEFFICIENT_RESTITUTION);
		} else if (mSurfaceHeight - b.mRadius < b.mPy) {
			b.mPy = mSurfaceHeight - b.mRadius;
			b.mDy = b.mDy * (-COEFFICIENT_RESTITUTION);
		}
	}
	
	/** {[NX */
	private class Ball {
		/** SW X */
		public float mPx;
		/** SW Y */
		public float mPy;
		/** x X */
		public float mDx;
		/** x Y */
		public float mDy;
		/** a */
		public float mRadius;
		/** `̃X^C */
		public Paint mPaint;

		/**
		 * w肳ꂽł̃IuWFNg𐶐܂
		 * 
		 * @param x SWX
		 * @param y SWY
		 * @param r a
		 * @param color F (16i 32bit ARGB)
		 */
		public Ball(float x, float y, float r, int color) {
			this.mPx = x;
			this.mPy = y;
			this.mRadius = r;

			mPaint = new Paint();
			mPaint.setColor(color);
			mPaint.setAntiAlias(true);
		}
	}
}
